import numpy as np
import SimpleITK as itk


def _get_metadata(reader):
    """ Read metadata from a SimpleITK.SimpleITK.ImageFileReader object.

    Args:
        reader (obj): SimpleITK file reader.

    Return:
        (dict): dicom tag and value pairs {tag: value}.
    """
    metadata = {}
    for tag in reader.GetMetaDataKeys():
        value = reader.GetMetaData(tag)
        metadata[tag] = value

    return metadata


def dcmread(dicom_path, read_header=False):
    """ Read 2D image data from the DICOM file.

    Args:
        dicom_path (str): path of the dicom file.
        read_header (bool): whether to return the dicom header together
            with the image array.

    Return:
        (ndarray): dicom image array.
        (dict, optional): dicom metadata.
    """
    img_itk = itk.ReadImage(dicom_path)
    img = itk.GetArrayFromImage(img_itk)
    img = np.squeeze(img)
    if read_header:
        metadata = _get_metadata(img_itk)
        return img, metadata
    else:
        return img


def dcminfo(dicom_path):
    """ Read metadata (dicom tags) from DICOM file.

    The metadata is stored in dicom tag and value pairs {tag: value}.
    For example, {'0008|0020': '20010316', '0018|0020': 'SE', ...}

    Args:
        dicom_path (str): path of the dicom file.

    Return:
        (dict): dicom metadata.
    """
    reader = itk.ImageFileReader()
    reader.SetFileName(dicom_path)
    reader.LoadPrivateTagsOn()
    reader.ReadImageInformation()

    metadata = _get_metadata(reader)
    return metadata



import ast
import numpy as np
import medvision as mv


def _parse_line(line, class2label):
    """ Parse a single line in detection dsmd.

    The single line content can be
    'data/1.png,170,146,397,681,cat'
    or
    'data/1.png,170,146,397,681,0.5,cat'
    or
    'data/1.png,,,,,'
    Note it can not be 'data/1.png'!

    Args:
        line (str): a single line content in detection dsmd.
        class2label (dict): the dict mapping class to label.
    Return:
        key (str): filename or filepath of an image.
        value (str): ground truth or detection result.
        label (int): the target label value.
    """
    key, value = line.strip().split(',', 1)
    value, class_name = value.strip().rsplit(',', 1)

    if class_name == '':
        return key, None, None
    else:
        value = ast.literal_eval(value.strip())
        return key, value, class2label[class_name]


def _update_data(data, line, class2label):
    """ Insert a single record into dataset metadata

    Args:
        data (dict): refer to 'load_det_dsmd' return value.
        line (str): a single line content in detection dsmd.
        class2label (dict): the dict mapping class to label.
    """
    num_classes = len(class2label)

    key, value, label = _parse_line(line, class2label)

    if label is None:
        data[key] = [None] * num_classes
        return

    if key not in data:
        data[key] = [None] * num_classes
        data[key][label] = [value]
    elif data[key][label] is None:
        data[key][label] = [value]
    else:
        data[key][label].append(value)


def _convert_bboxes_format(data):
    """ Convert bboxes from list of lists to ndarray of shape (n, 4).
    """
    assert len(data) != 0
    num_classes = len(next(iter(data.values())))

    # convert bboxes from list of lists to ndarray
    empty_bboxes = np.zeros((0, 4), dtype=np.float32)
    for key in data:
        for j in range(num_classes):
            if data[key][j] is not None:
                data[key][j] = np.array(data[key][j], dtype=np.float32)
            else:
                data[key][j] = empty_bboxes
    return data


def load_det_dsmd(dsmd_path, c2l_path):
    """ load detection dataset metadata.

    Args:
        dsmd_path (str): dataset metadata file path.
        c2l_path (str): class-to-label file.

    Return:
        (OrderedDict): Loaded dsmd is a OrderedDict looks like
        {
            data/1.png: [
                [bboxes of category 'cat' in ndarray of shape (n, 4)],
                [bboxes of category 'dog' in ndarray of shape (n, 4)],
                ...
            ]
            data/2.png: [
                ...
            ]
            ...
        }
    """
    data = {}
    class2label = mv.load_c2l(c2l_path)

    # label should start from 0
    assert min(class2label.values()) == 0

    with open(dsmd_path, 'r') as fd:
        for line in fd:
            _update_data(data, line, class2label)

    # convert bboxes from list of lists to ndarray
    data = _convert_bboxes_format(data)

    return mv.make_dsmd(data)


def _write_record(fd, key, value, label2class):
    """ Write a single record to dsmd file.

    Args:
        fd (file stream object): file stream.
        key (str): filename or filepath of an image.
        value (str): ground truth or detection result.
        label2class (dict): the dict mapping label to class name.
    """
    if sum([len(i) for i in value]) == 0:
        fd.write('%s,,,,,\n' % str(key))
        return
    else:
        for label, bboxes in enumerate(value):
            class_name = label2class[label]
            for bbox in bboxes:
                bbox = ','.join([str(elem) for elem in bbox])
                line = '%s,%s,%s\n' % (str(key), str(bbox), class_name)
                fd.write(line)


def save_det_dsmd(dsmd_path, data, c2l_path, auto_mkdirs=True):
    """ Save dataset metadata to specified file.

    Args:
        dsmd_path (str): file path to save dataset metadata.
        data (dict): dataset metadata, refer to 'load_dsmd'.
        c2l_path (str): class-to-label file.
        auto_mkdirs (bool): If the parent folder of `file_path` does
            not exist, whether to create it automatically.
    """
    if auto_mkdirs:
        mv.mkdirs(mv.parentdir(dsmd_path))

    # get label->class mapping
    class2label = mv.load_c2l(c2l_path)
    label2class = {value: key for key, value in class2label.items()}

    # write dataset metadata loop
    dsmd = mv.make_dsmd(data)
    with open(dsmd_path, 'w') as fd:
        for key, value in dsmd.items():
            _write_record(fd, key, value, label2class)



from .colorspace import (rgb2gray, gray2rgb)
from .geometry import (vflip, hflip, rot90, rotate, resize, rescale,
                       crop, center_crop, pad_to_square)
from .normalize import normalize_grayscale, normalize_rgb


__all__ = [
    'rgb2gray', 'gray2rgb',
    'vflip', 'hflip', 'rot90', 'rotate', 'resize', 'rescale', 'crop',
    'center_crop', 'pad_to_square',
    'normalize_grayscale', 'normalize_rgb'
]



from collections import OrderedDict
import random
from natsort import natsorted
import numpy as np
import medvision as mv


def make_dsmd(data):
    """ Make a dataset metadata.

    Args:
        data (dict): dataset metadata.
    """
    if isinstance(data, dict):
        dsmd = OrderedDict(natsorted(data.items()))
        return dsmd
    else:
        raise ValueError('dsmd only support dict type')


def split_dsmd_file(dsmd_filepath, datasplit, shuffle=True, suffix='.csv'):
    """ Split a dataset medadata file into 3 parts.

    Split a dataset metadata file into 'train.csv', 'val.csv' and 'test.csv'.
    And put them in the same directory with specified dsmd file.

    dsmd_filepath (str): file path of dataset metadata.
    datasplit (dict[str, float]): how to split the dataset. e.g.
        {'train': 0.9, 'val': 0.1, 'test': 0.0}
    shuffle (bool): whether to shuffle the dataset before splitting.

    Note:
        0.0 < datasplit['train'] + datasplit['val'] + datasplit['test'] <= 1.0
        If there's no image in a split. The corresponding dsmd file will
        not be saved.
    """
    dsmd_dir = mv.parentdir(dsmd_filepath)

    dsmd = mv.load_dsmd(dsmd_filepath)
    num_total = len(dsmd)

    keys = list(dsmd.keys())
    if shuffle:
        random.shuffle(keys)

    sum_ratio = 0.0
    splits = {}
    for mode, ratio in datasplit.items():
        file_path = mv.joinpath(dsmd_dir, mode + suffix)
        splits[file_path] = int(num_total * ratio)
        sum_ratio += ratio
    assert 0.0 < sum_ratio <= 1.0

    start_index = 0
    for file_path, num_cur_split in splits.items():
        end_index = start_index + num_cur_split

        start_index = np.clip(start_index, 0, num_total)
        end_index = np.clip(end_index, 0, num_total)

        keys_split = keys[start_index:end_index]
        keys_split = natsorted(keys_split)
        dsmd_split = {keys: dsmd[keys] for keys in keys_split}
        if len(dsmd_split) != 0:
            mv.save_dsmd(file_path, dsmd_split)
            mv.save_dsmd(file_path, dsmd_split)

        start_index = end_index


def match_dsmds(dt_src, gt_src, return_unmatched=False):
    """ Match 2 dataset metadata (e.g. annotation and detection result)

    Args:
        dt_src (dsmd): a dataset metadata instance.
        gt_src (dsmd): another dataset metadata instance.
        return_unmatched (bool): whether to return the unmatched
            part of dsmds.

    Return:
        (dsmd) matched dsmd of the first term.
        (dsmd) matched dsmd of the second term.
        (dsmd, optional) missing items in the 1st dsmd when
            matching with the 2nd dsmd.
        (dsmd, optional) unmatched items in the first dsmd when
            matching with the 2nd dsmd.
    """
    dt_keys = set(dt_src.keys())
    gt_keys = set(gt_src.keys())

    intersection_keys = dt_keys & gt_keys
    missing_keys = gt_keys - dt_keys
    unmatched_keys = dt_keys - gt_keys

    dt_dst = make_dsmd({key: dt_src[key] for key in intersection_keys})
    gt_dst = make_dsmd({key: gt_src[key] for key in intersection_keys})
    missing = make_dsmd({key: gt_src[key] for key in missing_keys})
    unmatched = make_dsmd({key: dt_src[key] for key in unmatched_keys})

    if return_unmatched:
        return dt_dst, gt_dst, missing, unmatched
    else:
        return dt_dst, gt_dst



from .fileutil import (isdir, isfile, listdir, joinpath, basename, parentdir,
                       cp, rm, cptree, rmtree, move, symlink, mkdirs,
                       empty_dir, non_overwrite_cp, copyfiles, GlobMode, glob,
                       find_duplicated_files)
from .multiprocessutil import tqdm_imap
from .typeutil import isarrayinstance


__all__ = [
    'isdir', 'isfile', 'listdir', 'joinpath', 'basename', 'parentdir', 'cp',
    'rm', 'cptree', 'rmtree', 'move', 'symlink', 'mkdirs', 'empty_dir',
    'non_overwrite_cp', 'copyfiles', 'GlobMode', 'glob',
    'find_duplicated_files',

    'isarrayinstance',
    'tqdm_imap'
]



import ast
import medvision as mv


def load_cls_dsmd(dsmd_path):
    data = {}
    with open(dsmd_path, 'r') as fd:
        for line in fd:
            key, value = line.strip().split(',', 1)
            try:  # try to interpret annotation as int or list[int].
                value = ast.literal_eval(value.strip())
            except (SyntaxError, ValueError):
                pass
            data[key] = value

    return mv.make_dsmd(data)


def save_cls_dsmd(dsmd_path, data, auto_mkdirs=True):
    if auto_mkdirs:
        mv.mkdirs(mv.parentdir(dsmd_path))

    dsmd = mv.make_dsmd(data)
    with open(dsmd_path, 'w') as fd:
        for key, value in dsmd.items():
            if mv.isarrayinstance(value):  # handle multi-label case
                value = ','.join([str(entry) for entry in value])
            line = '%s,%s\n' % (str(key), str(value))
            fd.write(line)


def gen_cls_dsmd_file_from_datafolder(
        root_dir, c2l_path, dsmd_path, classnames=None):
    """ Generate classification dataset metadata file from DataFolder for
    specified classes.

    DataFolder is a directory structure for image classification problems.
    Each sub-directory contains images from a special class. DataFolder
    directory structure looks like
    -----------------------
    ├── class1
    │   ├── 1.png
    │   └── 2.png
    │   ...
    ├── class2
    │   ├── 3.png
    │   └── 4.png
    └── ...
    -----------------------

    Args:
        root_dir (str): root data directory containing all the images.
        c2l_path (str): file path to save class2label info.
        dsmd_path (str): file path to save dataset metadata file.
        classnames (list[str]): names of specified classes.
            If not given, all classes are considered.

    Note:
        This function is expected to be used together with
        'gen_cls_ds_from_datafolder()'.
        Filename of each image in DataFolder should be unique. Otherwise,
        A FileExistsError will be thrown.
    """
    assert mv.isdir(root_dir)

    if classnames is None:
        classnames = mv.listdir(root_dir)

    class2label = {}
    dsmd = {}

    for label, classname in enumerate(classnames):
        class2label[classname] = label

        class_dir = mv.joinpath(root_dir, classname)
        assert mv.isdir(class_dir)
        filenames = mv.listdir(class_dir)
        for filename in filenames:
            if filename in dsmd:
                raise FileExistsError(
                    'filename {} already exists'.format(filename))
            dsmd[filename] = label

    mv.save_dsmd(c2l_path, class2label)
    mv.save_dsmd(dsmd_path, dsmd)


def gen_cls_ds_from_datafolder(
        in_dir, out_dir, auto_mkdirs=True, classnames=None):
    """ Generate classification dataset from DataFolder.

    This function will make a copy of each image in the DataFolder to the
    specified directory. Original DataFolder is left unchanged.

    Args:
        in_dir (str): DataFolder root directory.
        out_dir (str): directory to save all the images in DataFolder.
        auto_mkdirs (bool): If `out_dir` does not exist, whether to create
            it automatically.
        classnames (list[str]): names of specified classes to be collected.
            If not given, all classes are considered.

    Note:
        This function is expected to be used together with
        gen_cls_dsmd_file_from_datafolder().
        Filename of each image in DataFolder should be unique. Otherwise,
        A FileExistsError will be thrown.
        DataFolder is described in 'gen_cls_dsmd_file_from_datafolder()'.
    """
    assert mv.isdir(in_dir)

    # clean output directory
    if auto_mkdirs:
        mv.mkdirs(out_dir)
    mv.empty_dir(out_dir)

    if classnames is None:
        classnames = mv.listdir(in_dir)

    for classname in classnames:
        class_dir = mv.joinpath(in_dir, classname)
        assert mv.isdir(class_dir)
        filenames = mv.listdir(class_dir)
        mv.copyfiles(filenames, out_dir, class_dir, non_overwrite=True)



import matplotlib.pyplot as plt


def draw_froc(average_fps, sensitivity, save_path=None, **kwargs):
    """ Plot the FROC curve.

    Args:
        average_fps (list): A list containing the average number of false
        positives per image for different thresholds.
        sensitivity (list):  A list containing overall sensitivity for
        different thresholds.
        save_path (str): path to save the froc curve drawing.
    """
    plt.figure()
    plt.xlabel('False Positives')
    plt.ylabel('Sensitivity')
    plt.title('FROC Curve')
    plt.ylim([0, 1])
    plt.plot(average_fps, sensitivity, **kwargs)
    plt.grid()
    if save_path is not None:
        plt.savefig(save_path)
    plt.show()



import cv2
import numpy as np
import medvision as mv


def vflip(img):
    """ Flip an image vertically.

    Args:
        img (ndarray): image to be flipped.

    Returns:
        (ndarray): the vertically flipped image.
    """
    return np.ascontiguousarray(img[::-1, ...])


def hflip(img):
    """ Flip an image horizontally.

    Args:
        img (ndarray): image to be flipped.

    Returns:
        (ndarray): the horizontally flipped image.
    """
    return np.ascontiguousarray(img[:, ::-1, ...])


def rot90(img, k):
    """ Rotate 90 degrees.

    Rotate an array by 90 degrees for k times. Rotation direction is
    anticlockwise.

    Args:
        img (ndarray): image to be rotated.
        k (integer): number of times the array is rotated by 90 degrees.

    Returns:
        (ndarray): the rotated image.
    """
    return np.ascontiguousarray(np.rot90(img, k))


def rotate(src, angle, interpolation=cv2.INTER_LINEAR,
           border_mode=cv2.BORDER_REFLECT_101):
    """ Rotate an image by arbitrarily degrees.

    Perform arbitrary rotations on an image. The rotation center is the
    geometric center of the image. The shape of the image keeps unchanged
    after the rotation.

    Args:
        src (ndarray): image to be rotated.
        angle (float): rotation angle in degrees, positive values mean
            anticlockwise rotation.
        interpolation (int): interpolation method (opencv).
        border_mode (int): border interpolation mode (opencv).

    Returns:
        (ndarray): the rotated image.
    """
    height, width = src.shape[:2]
    center = ((width - 1) * 0.5, (height - 1) * 0.5)
    matrix = cv2.getRotationMatrix2D(center, angle, 1.0)
    dst = cv2.warpAffine(src, matrix, (width, height),
                         flags=interpolation, borderMode=border_mode)
    return dst


def resize(img, height, width, interpolation=cv2.INTER_LINEAR):
    """ Resize an image to the given size.

    Args:
        img (ndarray): the given image.
        height (int): target image height in pixel.
        width (int): target image width in pixel.
        interpolation (int): interpolation method (opencv).

    Returns:
        (ndarray): the resized image.
    """
    return cv2.resize(img, (width, height), interpolation=interpolation)


def rescale(src, scale, return_scale=False, interpolation=cv2.INTER_LINEAR):
    """ Resize image while keeping the aspect ratio.

    Args:
        src (ndarray): the input image.
        scale (float or tuple[int]): the scaling factor or maximum size.
            If it is a float number, then the image will be rescaled by this
            factor, else if it is a tuple of 2 integers, then the image will
            be rescaled as large as possible within the scale.
        return_scale (bool): whether to return the scaling factor besides the
            rescaled image.
        interpolation (int): interpolation method (opencv).

    Returns:
        (ndarray): the rescaled image
        (float, optional): the scaling factor
    """
    assert isinstance(scale, (float, int)) or mv.isarrayinstance(scale)

    height, width = src.shape[:2]

    # compute scale factor
    if isinstance(scale, (float, int)):
        assert scale > 0
    else:  # mv.isarrayinstance(scale):
        assert len(scale) == 2
        assert scale[0] > 0 and scale[1] > 0
        max_long_edge, max_short_edge = max(scale), min(scale)
        scale = min(max_long_edge / max(height, width),
                    max_short_edge / min(height, width))

    # rescale the image
    dst_height, dst_width = round(height * scale), round(width * scale)
    dst = resize(src, dst_height, dst_width, interpolation)

    return (dst, scale) if return_scale else dst


def crop(img, i, j, h, w):
    """ Crop an image.

    Args:
        img (numpy.ndarray): image to be cropped.
        i: upper pixel coordinate.
        j: left pixel coordinate.
        h: height of the cropped image.
        w: width of the cropped image.

    Returns:
        (ndarray): the cropped image.
    """
    return img[i:i+h, j:j+w, ...]


def center_crop(img, crop_height, crop_width):
    """ Crop the central part of an image.

    Args:
        img (ndarray): image to be cropped.
        crop_height (int): height of the crop.
        crop_width (int): width of the crop.

    Return:
        (ndarray): the cropped image.
    """
    def get_center_crop_coords(height, width, crop_height, crop_width):
        y1 = (height - crop_height) // 2
        y2 = y1 + crop_height
        x1 = (width - crop_width) // 2
        x2 = x1 + crop_width
        return x1, y1, x2, y2

    height, width = img.shape[:2]
    x1, y1, x2, y2 = get_center_crop_coords(
        height, width, crop_height, crop_width)
    return img[y1:y2, x1:x2, ...]


# def scaling_crop_pad(src, scale_factor, shift_factor,
#                      interpolation=cv2.INTER_LINEAR,
#                      border_mode=cv2.BORDER_REFLECT_101,
#                      pad_value=0):
#     """ Rescale an image and do cropping or padding.

#     Rescale an image according ot given scale factor. If scale factor > 1.0,
#     the rescaled image is cropped with the same size as the input image.
#     Otherwise, the rescaled image is padded to the size of the input image.

#     Args:
#         src (ndarray): input image.
#         scale_factor (float): scaling factor.
#         shift_factor (float): shift factor. It determines the
#         interpolation (int): interpolation method (opencv).
#         border_mode: border interpolation mode (opencv).
#         pad_value: values to be padded if
#             border_mode==cv2.BORDER_CONSTANT
#     """
#     dst = cv2.resize(src, None, fx=scale_factor, fy=scale_factor,
#                      interpolation=interpolation)

#     # resize image to original size with crop or pad
#     diff_y = abs(dst.shape[0] - src.shape[0])
#     diff_x = abs(dst.shape[1] - src.shape[1])

#     top = int(diff_y * shift_factor)
#     left = int(diff_x * shift_factor)
#     bottom = diff_y - top
#     right = diff_x - left

#     if scale_factor >= 1.0:
#         dst = dst[top:top+src.shape[0], left:left+src.shape[1], ...]
#     else:
#         if border_mode == cv2.BORDER_CONSTANT:
#             dst = cv2.copyMakeBorder(src, top, bottom, left,
#                                      right, border_mode, value=pad_value)
#         else:
#             dst = cv2.copyMakeBorder(src, top, bottom, left,
#                                      right, border_mode)

#     return dst


def pad_to_square(src, border_mode=cv2.BORDER_REFLECT_101, pad_value=0):
    """ Pad an image to so that its height and width are the same.

    For example, an image with shape (3, 4) will be padded to (4, 4).

    Args:
        src (ndarray): image to be padded.
        border_mode (int): border interpolation mode (opencv).
        pad_value(int): values to be padded if using
            border_mode==cv2.BORDER_CONSTANT
    """
    height, width = src.shape[:2]

    if height == width:
        return src

    sz = max(height, width)

    top = abs(sz - height) // 2
    bottom = abs(sz - height) - top
    left = abs(sz - width) // 2
    right = abs(sz - width) - left

    if border_mode == cv2.BORDER_CONSTANT:
        dst = cv2.copyMakeBorder(src, top, bottom, left,
                                 right, border_mode, value=pad_value)
    else:
        dst = cv2.copyMakeBorder(src, top, bottom, left, right, border_mode)

    return dst



from .draw_curve import draw_froc
from .image import Color, imshow, imshow_bboxes
from .visdom_visualizer import VisdomVisualizer
from .tensorboard_visualizer import TensorboardVisualizer

__all__ = [
    'draw_froc',
    'Color', 'imshow', 'imshow_bboxes',
    'VisdomVisualizer',
    'TensorboardVisualizer'
]



import collections
from enum import Enum, unique, auto
from glob import glob as std_glob
import hashlib
import os
import shutil
from natsort import natsorted
import medvision as mv


isdir = os.path.isdir
isfile = os.path.isfile


def listdir(path):
    return natsorted(os.listdir(path))


joinpath = os.path.join
basename = os.path.basename


def parentdir(path):
    path = os.path.abspath(path)
    return os.path.dirname(path)


cp = shutil.copy
rm = os.remove
cptree = shutil.copytree
rmtree = shutil.rmtree
move = shutil.move


def symlink(src, dst, overwrite=True, **kwargs):
    if os.path.lexists(dst) and overwrite:
        os.remove(dst)
    os.symlink(src, dst, **kwargs)


def mkdirs(path, mode=0o777):
    path = os.path.expanduser(path)
    path = os.path.abspath(path)
    os.makedirs(path, mode, exist_ok=True)


def empty_dir(path):
    assert isdir(path)
    rmtree(path)
    mkdirs(path)


def non_overwrite_cp(src, dst):
    if isfile(dst):
        raise FileExistsError('target file {} already exists'.format(dst))

    if isdir(dst):
        filename = basename(src)
        dst_filepath = joinpath(dst, filename)
        if isfile(dst_filepath):
            raise FileExistsError(
                'target file {} already exists'.format(dst_filepath))

    return cp(src, dst)


def copyfiles(src_paths, dst_dir, src_root=None, non_overwrite=False):
    assert isdir(dst_dir)
    assert mv.isarrayinstance(src_paths)

    cp_func = non_overwrite_cp if non_overwrite else cp
    for src_path in src_paths:
        cp_func(joinpath(src_root, src_path), dst_dir)


@unique
class GlobMode(Enum):
    FILE = auto()
    DIR = auto()
    ALL = auto()


def glob(root, pattern='*', mode=GlobMode.FILE, recursive=False):
    root = os.path.expanduser(root)
    pattern = joinpath('**', pattern) if recursive else pattern
    pathname = joinpath(root, pattern)

    result = std_glob(pathname, recursive=recursive)

    if mode == GlobMode.FILE:
        result = filter(isfile, result)
    elif mode == GlobMode.DIR:
        result = filter(isdir, result)
    else:  # GlobMode.ALL
        pass

    return natsorted(result)


def compute_md5_str(file_path):
    if not mv.isfile(file_path):
        return None

    with open(file_path, 'rb') as f:
        m = hashlib.md5()
        m.update(f.read())
        md5_code = m.hexdigest()
        return str(md5_code).lower()


def find_duplicated_files(data_dir, pattern='*'):
    """ Find duplicated files in specified directory.

    Args:
        data_dir (str): specified directory to be scanned.
        pattern: refer to 'glob()'.

    Return:
        (list[tuple]): duplicated file path pairs.
    """
    filepaths = glob(data_dir, pattern, mode=GlobMode.FILE, recursive=True)
    md5s = [compute_md5_str(filepath) for filepath in filepaths]
    md5_counts = collections.Counter(md5s)

    duplicated_files = []
    for key, count in md5_counts.items():
        if count > 1:
            candidates = tuple(filepaths[i] for
                               i, x in enumerate(md5s) if x == key)
            duplicated_files.append(candidates)

    return duplicated_files



# flake8: noqa
from .__version__ import __version__
from .dataset import *
from .dicom import *
from .evaluation import *
from .image import *
from .util import *
from .torch_util import *
from .visualization import *
